// ignore: implementation_imports
import 'package:build/src/builder/build_step.dart';
import 'package:analyzer/dart/element/element.dart' as ele;
import 'package:source_gen/source_gen.dart';
import 'package:build/build.dart';
import 'dart:io';
import 'tpl.dart' as tpl;
import 'assets.dart';

class AssetsGenerator extends GeneratorForAnnotation<GAssets> {
  final separator = Platform.pathSeparator;
  final List<String> assetsList = [];
  List ignores = [
//    '.json',
    '.ttf',
    '.DS_Store',
  ];

  String buildAssets(list) {
    var sb = StringBuffer();
    list.forEach((value) {
      sb.writeln("    - $value");
    });
    return sb.toString();
  }

  String buildCode(List<String> list, assetsPath) {
    var sb = StringBuffer();
    list.forEach((value) {
      String arr1Last = value.split('- $assetsPath/').last;
      List<String> arr2 = arr1Last.split('/');
      arr2.removeAt(0);
      arr2[arr2.length - 1] = arr2[arr2.length - 1].split('.').first;
      sb.writeln("  static const String ${arr2.join('_')} = '$arr1Last';");
    });
    return sb.toString();
  }

  String readPubspec(String path, List list, assetsPath) {
    var sb = StringBuffer();
    try {
      List<String> lines = File(path).readAsLinesSync();
      if (list == null || list.isEmpty) {
        return File(path).readAsStringSync();
      }
      bool one = true;
      lines.forEach((v) {
        if (v.indexOf(RegExp(r".*assets:")) >= 0 && one) {
          one = false;
          sb.writeln('  assets:');
          sb.writeln(buildAssets(list));
        } else {
          var nv =
              v.replaceAll(RegExp("((.*- $assetsPath/(.*))|(.*assets:))"), "");
          if (nv.trim().length > 0) {
            sb.writeln(nv);
          }
        }
      });
      return sb.toString();
    } catch (e) {
      stdout.writeln('文件读取失败! $e');
    }
    return sb.toString();
  }

  void readFiles(String path, String package) {
    try {
      Directory(path).listSync().forEach((f) {
        var bool = FileSystemEntity.isFileSync(f.path);
        if (bool) {
          String filePath = f.path.split('/$package/').last;

          ///忽略自动生成代码
          var ignoreItem;
          try {
            ignoreItem =
                ignores.firstWhere((element) => f.path.endsWith(element));
          } catch (e) {}
          if (ignoreItem == null) {
            assetsList.add(filePath);
          }
        } else {
          readFiles(f.path, package);
        }
      });
    } catch (e) {
      stdout.writeln('目录不存在! $e');
    }
  }

  @override
  generateForAnnotatedElement(
      ele.Element element, ConstantReader annotation, BuildStep buildStep) {
    var assetsPath = '';
    var enable = false;
    try {
      enable = annotation.read('enable')?.boolValue;
    } catch (e) {}
    if (!enable) return null;
    try {
      assetsPath = annotation.read('assetsPath')?.stringValue;
    } catch (e) {}
    if (assetsPath.length <= 0) {
      return null;
    }
    Directory('${Directory.current.path}$separator$assetsPath')
        .createSync(recursive: true);
    readFiles('${Directory.current.path}$separator$assetsPath',
        buildStep.inputId.package);

    File('${Directory.current.path}${separator}pubspec.yaml').writeAsStringSync(
        readPubspec('${Directory.current.path}${separator}pubspec.yaml',
            assetsList, assetsPath));

    Directory('${Directory.current.path}${separator}lib${separator}common')
        .createSync(recursive: true);
    File('${Directory.current.path}${separator}lib${separator}common${separator}assets_constant.assets.dart')
        .writeAsStringSync(
            tpl.clazzTpl(buildCode: buildCode(assetsList, assetsPath)));
    return null;
  }
}

Builder assetsBuilder(BuilderOptions options) =>
    LibraryBuilder(AssetsGenerator(), generatedExtension: ".assets.dart");
