// ignore: implementation_imports
import 'package:build/src/builder/build_step.dart';
import 'package:analyzer/dart/element/element.dart' as ele;
import 'package:source_gen/source_gen.dart';
import 'package:build/build.dart';
import 'dart:convert';
import 'dart:io';
import 'iconfont.dart';
import 'tpl/tpl.dart' as tpl;
import 'tpl/tpl_widget.dart';

class IconfontGenerator extends GeneratorForAnnotation<GIconfont> {
  final separator = Platform.pathSeparator;
  List ignores = [
    '.json',
    '.ttf',
    '.svg',
  ];
  Map<String, List<String>> svgMap = Map();

  String buildCode(path) {
    var sb = StringBuffer();
    try {
      var newPath = path
          .split('/')
          .where((element) => (element != null && element.length > 0))
          .join('/');
      var content = File(
              '${Directory.current.path}$separator$newPath${separator}iconfont.json')
          .readAsStringSync();
      Map map = json.decode(content);
      map.forEach((key, value) {
        if (value is List) {
          value.forEach((ele) {
            sb.writeln('''  ///iconfont name ${ele['name'] ?? ''}''');
            sb.writeln(
                '''  static const IconData ${(ele['font_class'] as String).toUpperCase()} = const IconData(0x${ele['unicode']}, fontFamily: 'iconfont');''');
          });
        }
      });
    } catch (e) {}
    return sb.toString();
  }

  String readPubspec(String path, String iconfontPath) {
    var sb = StringBuffer();
    var newPath = iconfontPath
        .split('/')
        .where((element) => (element != null && element.length > 0))
        .join('/');
    try {
      List<String> lines = File(path).readAsLinesSync();
      bool one = true;
      lines.forEach((v) {
        if (v.indexOf(RegExp(r".*fonts:")) >= 0 && one) {
          one = false;
          sb.writeln('  fonts:');
          sb.writeln('    - family: iconfont');
          sb.writeln('      fonts:');
          sb.writeln('        - asset: $newPath${separator}iconfont.ttf');
        } else {
          if (!(v.indexOf(RegExp(
                  r"((- family:)|(fonts:)|(- asset:)|(style:)|(weight:))")) >=
              0)) {
            sb.writeln(v);
          }
        }
      });
      return sb.toString();
    } catch (e) {
      stdout.writeln('文件读取失败! $e');
    }
    return sb.toString();
  }

  Future<String> httpRead(String url) async {
    var newUrl = url;
    if (!newUrl.startsWith('https')) {
      newUrl = 'https:$url';
    }
    var httpClient = HttpClient();
    String result = '';
    try {
      var request = await httpClient.getUrl(Uri.parse(newUrl));
      var response = await request.close();
      if (response.statusCode == HttpStatus.ok) {
        result = await response.transform(utf8.decoder).join();
      }
    } catch (exception) {}
    return result;
  }

  String netJSSvgParse(String input, RegExp exp) {
    Iterable<Match> matches = exp.allMatches(input);
    String output = '';
    for (Match m in matches) {
      String match = m.group(0);
      output += match;
    }
    return output;
  }

  List<String> netJSSvgParseList(String input, RegExp exp) {
    Iterable<Match> matches = exp.allMatches(input);
    List<String> output = [];
    for (Match m in matches) {
      String match = m.group(0);
      output.add(match);
    }
    return output;
  }

  String buildIconName() {
    var sb = StringBuffer();
    svgMap.keys.toList().forEach((element) {
      sb.writeln('''  ${element.toUpperCase()},''');
    });
    return sb.toString();
  }

  String buildSvgCase() {
    var sb = StringBuffer();
    svgMap.forEach((key, value) {
      sb.writeln("""      case IconFontName.${key.toUpperCase()}:""");
      sb.writeln("""        svgXml = '''""");
      sb.writeln(
          """          <svg viewBox="0 0 1024 1024" xmlns="http://www.w3.org/2000/svg">""");
      for (int i = 0; i < value.length; i++) {
        String path = value[i];
        // 取出path中的fill
        String fillColor = netJSSvgParse(path, RegExp(r'(?<=fill=").*?(?=")'));
        if (fillColor.isNotEmpty) {
          path = path.replaceAll(
              fillColor, "\${getColor($i, color, colors, '$fillColor')}");
        } else {
          path = path.replaceAll('''fill=""''', '');
          path = path.replaceAll("""fill=''""", '');
          path = path.replaceAllMapped(RegExp(r'></path>'), (match) {
            return ''' fill="\${getColor($i, color, colors, '$fillColor')}"></path>''';
          });
        }
        sb.writeln("""            $path""");
      }
      sb.writeln("""          </svg>""");
      sb.writeln("""        ''';""");
      sb.writeln("""        break;""");
    });
    return sb.toString();
  }

  @override
  generateForAnnotatedElement(ele.Element element, ConstantReader annotation,
      BuildStep buildStep) async {
    var enable = false;
    try {
      enable = annotation.read('enable')?.boolValue;
    } catch (e) {}
    if (!enable) return null;
    String path = '', networkUrl = '', prefix = 'icon';
    try {
      path = annotation.read('path')?.stringValue;
    } catch (e) {}
    try {
      networkUrl = annotation.read('networkUrl')?.stringValue;
    } catch (e) {}
    try {
      prefix = annotation.read('prefix')?.stringValue;
    } catch (e) {}

    if (path.isNotEmpty) {
      File('${Directory.current.path}${separator}pubspec.yaml')
          .writeAsStringSync(readPubspec(
              '${Directory.current.path}${separator}pubspec.yaml', path));

      Directory('${Directory.current.path}${separator}lib${separator}common')
          .createSync(recursive: true);
      File('${Directory.current.path}${separator}lib${separator}common${separator}iconfont_constant.iconfont.dart')
          .writeAsStringSync(tpl.clazzTpl(buildCode: buildCode(path)));
    }
    if (networkUrl.isNotEmpty) {
      String result = await httpRead(networkUrl);
      if (result.isNotEmpty) {
        // 第一步取出<svg></svg> 标签对里内容
        String parse1 =
            netJSSvgParse(result, RegExp(r'(?<=<svg>).*?(?=</svg>)'));
        if (parse1.isNotEmpty) {
          // 第二步取出<symbol></symbol> 标签对
          List<String> parse2 =
              netJSSvgParseList(parse1, RegExp(r'<symbol(.*?)</symbol>'));
          // 第三步取出name, 取出path
          parse2.forEach((element) {
            String name =
                netJSSvgParse(element, RegExp('''(?<=id="$prefix).*?(?=")'''));
            // 处理非法字符
            name =
                name.replaceAllMapped(RegExp(r'[^a-zA-Z0-9_]'), (match) => '_');
            List<String> pathList =
                netJSSvgParseList(element, RegExp(r'<path(.*?)</path>'));
            svgMap[name] = pathList;
          });
        }
      }
      if (svgMap.isNotEmpty) {
        String nameCode = buildIconName();
        String svgCase = buildSvgCase();
        String code = clazzTplWidget(nameCode: nameCode, svgCase: svgCase);
        // 写文件
        Directory('${Directory.current.path}${separator}lib${separator}widget')
            .createSync(recursive: true);
        File('${Directory.current.path}${separator}lib${separator}widget${separator}icon_font.iconfont.dart')
            .writeAsStringSync(code);
      }
    }
    return null;
  }
}

Builder iconfontBuilder(BuilderOptions options) =>
    LibraryBuilder(IconfontGenerator(), generatedExtension: ".iconfont.dart");
