// ignore: implementation_imports
import 'package:build/src/builder/build_step.dart';
import 'package:analyzer/dart/element/element.dart' as ele;
import 'package:source_gen/source_gen.dart';
import 'package:build/build.dart';
import 'dart:io';
import 'dart:convert';
import 'locale.dart';
import 'tpl.dart' as tpl;

class LocaleGenerator extends GeneratorForAnnotation<GLocale> {
  final separator = Platform.pathSeparator;

  Map<String, dynamic> localeMap = {};
  Map<String, dynamic> effectiveMap = {};

  Map<String, String> localeRegs = {
    'local': r"I18N\.local[\(][^\(\)\{\}]*[\)]",
    'localMap':
        r"Map<String, Map<String, String>> _LOCALIZATION_MAP = (\s+)?[\{][^()]*[\}];",
    'localGet':
        r"String(\s+)?get(\s+)?(.*)CustomLocalizations\.of\(this\)\._localeValue\(key: (.*)\);",
    'localCode': r"I18N\.[^,][\w]+",
  };
  List ignores = [
    '.local.dart',
  ];
  List<Map<String, String>> escapeChars = [
    {'key': '\n', 'keyR': r'\n', 'value': '\\n', 'valueR': r'\\n'},
    {'key': '\t', 'keyR': r'\t', 'value': '\\t', 'valueR': r'\\t'},
    {'key': '\r', 'keyR': r'\r', 'value': '\\r', 'valueR': r'\\r'},
  ];

  void readFile(path) {
    try {
      File file = File(path);
      bool exists = file.existsSync();
      if (exists) {
        String code = file.readAsStringSync();
        RegExp(localeRegs['localMap']).allMatches(code).forEach((v) {
          String cStr = v.group(0);
          cStr = cStr.replaceAll(
              'Map<String, Map<String, String>> _LOCALIZATION_MAP = ', '');
          cStr = cStr.substring(0, cStr.length - 1);
          cStr = cStr.replaceAllMapped(RegExp(r'\}[[,][\s\S]]*[\w\W]?\}'),
              (match) {
            return match.group(0).replaceAll(',', '');
          });
          if (cStr != null && cStr.isNotEmpty) {
            localeMap = jsonDecode(cStr, reviver: (key, value) {
              if (value is String) {
                escapeChars.forEach((map) {
                  value =
                      (value as String).replaceAll(map['key'], map['valueR']);
                });
              }
              return value;
            });
          }
        });
      }
    } catch (e) {
      stdout.writeln('读取文件失败! $path $e');
    }
  }

  updateFiles(String root, {package}) async {
    try {
      var directory = Directory(root);
      List<FileSystemEntity> files = directory.listSync();
      for (FileSystemEntity f in files) {
        var bool = FileSystemEntity.isFileSync(f.path);
        if (!bool) {
          await updateFiles(f.path, package: package);
        } else {
          ///之后可以通过文件hash 来判断文件是否被变化，提高效率
          var ignoreItem;
          try {
            ignoreItem =
                ignores.firstWhere((element) => f.path.endsWith(element));
          } catch (e) {}
          if (ignoreItem == null && f.path.endsWith('.dart')) {
            await updateLocaleLine(f.path, package: package);
          }
        }
      }
    } catch (e) {
      stdout.writeln("目录不存在！$root");
    }
  }

  updateLocaleLine(path, {package}) async {
    try {
      File file = File(path);
      String code = file.readAsStringSync();
      String newCode = code;
      Map<String, String> update = {};

      RegExp(localeRegs['localCode']).allMatches(code).forEach((v) {
        String cStr = v.group(0);
        String key = cStr.substring('I18N.'.length);
        if (localeMap[key] != null) {
          // 有效
          effectiveMap[key] = localeMap[key];
        }
      });

      bool flag = false;
      String importStr =
          "import 'package:$package/common/localization_constant.local.dart';\r\n";
      RegExp(localeRegs['local']).allMatches(code).forEach((v) {
        if (!newCode.contains(importStr) && !flag) {
          flag = true;
          newCode = importStr + newCode;
        }
        String localCode = v.group(0);
        RegExp reg = RegExp(r"I18N\.local[\(]");
        List<String> localArr = localCode.split(reg);
        localArr[0] = reg.firstMatch(localCode).group(0);
        localCode =
            localArr[1].trim().substring(0, localArr[1].trim().length - 1);
        List<String> arr =
            localCode.split(RegExp(r"(\s+)?,(\s+)?((en:)|(zh:))"));
        var index = localCode.indexOf(RegExp(r"en:[\s\S]*zh:"));
        if (index < 0) {
          arr = [arr[0], arr[2], arr[1]];
        }
        for (var i = 0; i < arr.length; i++) {
          arr[i] = arr[i].trim().replaceAll(RegExp(r'''^"""|"""$'''), '"');
          arr[i] = arr[i].trim().replaceAll(RegExp(r"""^'''|'''$"""), '"');
        }
        if (arr.isNotEmpty) {
          var strKey = arr[0];
          String key = strKey.substring(1, strKey.length - 1);
          String en = arr[1].substring(1, arr[1].length - 1);
          String zh = arr[2].substring(1, arr[2].length - 1);
          // 处理\n
          escapeChars.forEach((element) {
            en = en.replaceAll(element['key'], element['valueR']);
            zh = zh.replaceAll(element['key'], element['valueR']);
          });
          effectiveMap[key] = {"en": en, "zh": zh};
          update[v.group(0)] = '''I18N.$key''';
        }
      });

      update.forEach((key, value) {
        newCode = newCode.replaceAll(key, value);
      });

      ///写入文件
      await file.writeAsString(newCode);
    } catch (e) {
      stdout.writeln('读取文件失败! $path $e');
    }
  }

  String buildGet(Map<String, dynamic> map) {
    var sb = StringBuffer();
    map.forEach((key, value) {
      sb.writeln("""  // ignore: non_constant_identifier_names""");
      sb.writeln(
          """  static String get $key => CustomLocalizations.of(null)._localeValue(key: '$key');""");
    });
    return sb.toString();
  }

  String buildMap(Map<String, dynamic> map) {
    var sb = StringBuffer();
    map.forEach((key, value) {
      var en = value['en'];
      var zh = value['zh'];
      escapeChars.forEach((map) {
        en = (en as String).replaceAll(map['value'], map['valueR']);
        zh = (zh as String).replaceAll(map['value'], map['valueR']);
      });
      sb.writeln('''  "$key": {"en": """$en""", "zh": """$zh"""},''');
    });
    return sb.toString();
  }

  String buildEscape(List<Map<String, String>> list) {
    var sb = StringBuffer();
    sb.writeln('''    map.forEach((key, value) {''');
    sb.writeln('''      map[key] = value.map((key, value) {''');
    list.forEach((element) {
      sb.writeln(
          '''        value = value.replaceAll('${element['valueR']}', '${element['keyR']}');''');
    });
    sb.writeln('''        return MapEntry(key, value);''');
    sb.writeln('''      });''');
    sb.writeln('''    });''');
    return sb.toString();
  }

  String buildEscapeCode(List<Map<String, String>> list) {
    var sb = StringBuffer();
    list.forEach((element) {
      sb.writeln(
          '''    value = value.replaceAll('${element['valueR']}', '${element['keyR']}');''');
    });
    return sb.toString();
  }

  @override
  generateForAnnotatedElement(ele.Element element, ConstantReader annotation,
      BuildStep buildStep) async {
    var init = false;
    var enable = false;
    try {
      enable = annotation.read('enable')?.boolValue;
    } catch (e) {}
    if (!enable) return null;
    try {
      init = annotation.read('init')?.boolValue;
    } catch (e) {}
    if (!init) {
      return null;
    }
    if (init) {
      //生成文件
      var exists = Directory(
              '${Directory.current.path}${separator}lib${separator}common')
          .existsSync();
      if (exists) {
        var file = File(
            '${Directory.current.path}${separator}lib${separator}common${separator}localization_constant.local.dart');
        if (!file.existsSync()) {
          //文件不存在，生成
          File('${Directory.current.path}${separator}lib${separator}common${separator}localization_constant.local.dart')
              .writeAsStringSync(tpl.clazzTpl(buildGet: '', buildMap: ''));
        }
      } else {
        //不存在则生成
        Directory('${Directory.current.path}${separator}lib${separator}common')
            .createSync(recursive: true);
        File('${Directory.current.path}${separator}lib${separator}common${separator}localization_constant.local.dart')
            .writeAsStringSync(tpl.clazzTpl(buildGet: '', buildMap: ''));
      }
    }
    //读取已经生成的文件配置
    readFile(
        '${Directory.current.path}${separator}lib${separator}common${separator}localization_constant.local.dart');
    //查找local并修改文件
    await updateFiles('${Directory.current.path}${separator}lib',
        package: buildStep.inputId.package);
    //生成Local class
    Directory('${Directory.current.path}${separator}lib${separator}common')
        .createSync(recursive: true);
    File('${Directory.current.path}${separator}lib${separator}common${separator}localization_constant.local.dart')
        .writeAsStringSync(tpl.clazzTpl(
      package: buildStep.inputId.package,
      buildGet: buildGet(effectiveMap),
      buildMap: buildMap(effectiveMap),
      escapeChars: buildEscape(escapeChars),
      escapeCode: buildEscapeCode(escapeChars),
    ));
    return null;
  }
}

Builder localeBuilder(BuilderOptions options) =>
    LibraryBuilder(LocaleGenerator(), generatedExtension: ".locale.dart");
