// ignore: implementation_imports
import 'package:build/src/builder/build_step.dart';
import 'package:analyzer/dart/element/element.dart' as ele;
import 'package:source_gen/source_gen.dart';
import 'package:build/build.dart';
import 'dart:io';
import 'project.dart';
import 'tpl/tpl.dart' as tpl;

class ProjectGenerator extends GeneratorForAnnotation<GProject> {
  final separator = Platform.pathSeparator;

  String readPubspec(String path) {
    var sb = StringBuffer();
    try {
      String content = File(path).readAsStringSync();
      String newContent = content.replaceFirst(RegExp(r"""  flutter:
    sdk: flutter"""), """  flutter:
    sdk: flutter
  flutter_localizations:
    sdk: flutter
  flutter_cupertino_localizations: ^1.0.1
  redux: ^4.0.0+3
  flutter_redux: ^0.7.0
  redux_thunk: ^0.3.0
  redux_logging: ^0.4.0
  redux_persist_flutter: ^0.8.3
  dio: ^3.0.10""");
      return newContent.toString();
    } catch (e) {
      stdout.writeln('文件读取失败! $e');
    }
    return sb.toString();
  }

  @override
  generateForAnnotatedElement(
      ele.Element element, ConstantReader annotation, BuildStep buildStep) {
    var init = false;
    var enable = false;
    try {
      enable = annotation.read('enable')?.boolValue;
    } catch (e) {}
    if (!enable) return null;
    try {
      init = annotation.read('init')?.boolValue;
    } catch (e) {}
    if (!init) {
      return null;
    }
    var package = buildStep.inputId.package;
    //初始化第三方库. build.sh
    File('${Directory.current.path}${separator}pubspec.yaml').writeAsStringSync(
        readPubspec('${Directory.current.path}${separator}pubspec.yaml'));
    File('${Directory.current.path}${separator}build.sh')
      ..createSync()
      ..writeAsStringSync(tpl.Tpl.buildSh(package: package));
    //删除main.dart, test
    File('${Directory.current.path}${separator}lib${separator}main.dart')
        .deleteSync();
    File('${Directory.current.path}${separator}test')
        .deleteSync(recursive: true);
    tpl.Tpl.buildCode.forEach((info) {
      var filePath =
          '${Directory.current.path}${separator}lib$separator${info.filePath}';
      File(filePath)
        ..createSync(recursive: true)
        ..writeAsStringSync(info.codeBuild(package: package));
    });
    return null;
  }
}

Builder projectBuilder(BuilderOptions options) =>
    LibraryBuilder(ProjectGenerator(), generatedExtension: ".project.dart");
