String clazzTpl({package}) => """
import 'package:$package/widget/base_store_container.dart';
import 'package:flutter/material.dart';
import 'package:flutter_localizations/flutter_localizations.dart';
import 'package:flutter_redux/flutter_redux.dart';
import 'package:redux/redux.dart';
import 'package:xanno/xanno.dart';
import 'common/localization_constant.local.dart';
import 'debug/app_debug_model.dart';
import 'route/main.route.dart';
import 'store/main.dart';
import 'utils/print_util.dart';

@GApi(init: true)
@GColor(init: true)
@GSize(init: true)
@GLocale(init: true, enable: false)
@GAssets(assetsPath: 'assets')
@GFormatter(auto: true)
//@GIconfont(
//    prefix: '', networkUrl: '//at.alicdn.com/t/font_xxxx.js')
class App extends StatefulWidget {
  final Store<AppState> store;
  final String route;

  App({Key key, this.store, this.route}) : super(key: key);

  @override
  _AppState createState() => _AppState();
}

class _AppState extends State<App> {
  @override
  Widget build(BuildContext context) {
    return StoreProvider<AppState>(
      store: widget.store,
      child: BaseStoreContainer(
        builder: (context, store) {
          return MaterialApp(
            localizationsDelegates: [
              CustomLocalizationDelegate(),
              GlobalMaterialLocalizations.delegate,
              GlobalWidgetsLocalizations.delegate,
              GlobalCupertinoLocalizations.delegate,
            ],
            supportedLocales: [
              CustomLocale.en,
              CustomLocale.zh,
            ],
            localeListResolutionCallback:
                (List<Locale> locales, Iterable<Locale> supportedLocales) {
              bool zh = locales.first.languageCode.contains('zh');
              return zh ? CustomLocale.zh : CustomLocale.en;
            },
            theme: ThemeData(
              platform: TargetPlatform.iOS,
              brightness: Theme.of(context).brightness,
              splashColor: Colors.transparent,
              highlightColor: Colors.transparent,
              primaryColorBrightness: Brightness.light,
            ),
            navigatorKey: globalNavigatorKey,
            debugShowCheckedModeBanner: context.isOn(AppDebugKey.debugBanner),
            initialRoute: RouteConstant.MainPage,
            routes: routes(context),
            navigatorObservers: [
              NavObserver(),
            ],
          );
        },
      ),
    );
  }
}

class NavObserver extends NavigatorObserver {
  @override
  void didPush(Route route, Route previousRoute) {
    logDebugPrint('push \${route.settings.name}');
  }

  @override
  void didPop(Route route, Route previousRoute) {
    logDebugPrint('pop \${route.settings.name}');
  }

  @override
  void didReplace({Route newRoute, Route oldRoute}) {
    logDebugPrint('replace \${newRoute.settings.name}');
  }
}
""";
