String clazzTpl({package}) => """
import 'dart:async';
import 'dart:io';
import 'package:flutter/material.dart';
import 'dart:ui';
import 'package:flutter/services.dart';
import 'package:redux/redux.dart';
import 'package:redux_persist/redux_persist.dart';
import 'package:redux_persist_flutter/redux_persist_flutter.dart';
import 'app.dart';
import 'store/main.dart';
import 'store/reducers.dart';
import 'store/redux_middleware.dart';
import 'package:path_provider/path_provider.dart';

void main() async {
  FlutterError.onError = (FlutterErrorDetails details) async {
    Zone.current.handleUncaughtError(details.exception, details.stack);
  };

  void errorLog(Object error, StackTrace stackTrace) async {
    StringBuffer sb = StringBuffer();
    sb.writeln('error=================');
    sb.writeln(error.toString());
    sb.writeln('stackTrace=================');
    sb.writeln(stackTrace.toString());
    String dir = (await getApplicationDocumentsDirectory()).path;
    var now = DateTime.now();
    String fileName =
        '\${now.year}年\${now.month}月\${now.day}日\${now.hour}时\${now.minute}分\${now.second}秒';
    await File('\$dir\${Platform.pathSeparator}\$fileName.log')
        .writeAsString(sb.toString());
  }

  runZonedGuarded<Future<void>>(() async {
    WidgetsFlutterBinding.ensureInitialized();
    final persistor = Persistor<AppState>(
      storage: FlutterStorage(key: '${package}_store_v1'),
      serializer: JsonSerializer<AppState>(AppState.fromJson),
    );

    final initialState = await persistor.load();

    final store = Store<AppState>(
      appReducer,
      middleware: [persistor.createMiddleware(), ...userMiddleware],
      initialState: initialState ?? AppState.initialState(),
    );

    SystemChrome.setSystemUIOverlayStyle(SystemUiOverlayStyle(
        statusBarColor: Colors.transparent,
        systemNavigationBarColor: Colors.transparent,
        systemNavigationBarDividerColor: Colors.transparent));
    runApp(App(store: store, route: window.defaultRouteName));
  }, errorLog);
}
""";
