// ignore: implementation_imports
import 'package:build/src/builder/build_step.dart';
import 'package:analyzer/dart/element/element.dart' as ele;
import 'route.dart';
import 'package:source_gen/source_gen.dart';
import 'package:build/build.dart';
import 'dart:io';
import 'tpl/tpl.dart' as tpl;
import 'tpl/tpl_guard.dart' as tpl_guard;

class RouteGenerator extends GeneratorForAnnotation<GRoute> {
  final separator = Platform.pathSeparator;
  final Map constantMap = Map();
  final Map routesMap = Map();
  final Map<String, bool> guardMap = Map();
  final Map importMap = Map();
  var defaultRoute = "";
  var debugRoute = "";

  String buildConstant(map, defaultRoute, debugRoute) {
    var sb = StringBuffer();
    map.forEach((key, value) {
      if (value == defaultRoute) {
        sb.writeln("  /// 入口页面");
      } else if (value == debugRoute) {
        sb.writeln("  /// debug页面");
      }
      sb.writeln("  static const String $value = '$key';");
    });
    return sb.toString();
  }

  String buildWidgetName(map) {
    var sb = StringBuffer();
    map.forEach((key, value) {
      sb.writeln("    RouteConstant.$value: '$value',");
    });
    return sb.toString();
  }

  String buildRoutes(map, defaultRoute, guardMap) {
    var sb = StringBuffer();
    map.forEach((key, value) {
      if (key == defaultRoute) {
        sb.writeln("    RouteConstant.$key: (context) {");
        sb.writeln("      ScreenSizeFit.init(context);");
        if (guardMap[key]) {
          //需要支持路由守卫
          sb.writeln("      return GuardWidget(child: $key(title: '$value'));");
        } else {
          sb.writeln("      return $key(title: '$value');");
        }
        sb.writeln("    },");
      } else {
        if (guardMap[key]) {
          //需要支持路由守卫
          sb.writeln(
              "    RouteConstant.$key: (context) => GuardWidget(child: $key(title: '$value')),");
        } else {
          sb.writeln(
              "    RouteConstant.$key: (context) => $key(title: '$value'),");
        }
      }
    });
    return sb.toString();
  }

  String buildImport(map) {
    var sb = StringBuffer();
    //添加同级route_guard
    if (guardMap.containsValue(true)) {
      sb.writeln("import 'route_guard.route.dart';");
    }
    map.forEach((key, value) {
      sb.writeln("import '$key';");
    });
    return sb.toString();
  }

  @override
  generateForAnnotatedElement(
      ele.Element element, ConstantReader annotation, BuildStep buildStep) {
    var enable = false;
    try {
      enable = annotation.read('enable')?.boolValue;
    } catch (e) {}
    if (!enable) return null;
    var url = annotation.read('url')?.stringValue;
    var title = annotation.read('title')?.stringValue;
    var guard = false;
    try {
      guard = annotation.read('guard')?.boolValue;
    } catch (e) {}
    if (url == '/') {
      //入口
      defaultRoute = element.name;
    } else if (url == '/debug') {
      debugRoute = element.name;
    }
    constantMap[url] = element.name;
    routesMap[element.name] = title;
    guardMap[element.name] = guard;
    if (buildStep.inputId.path.contains('lib/')) {
      importMap[
              "package:${buildStep.inputId.package}/${buildStep.inputId.path.replaceFirst('lib/', '')}"] =
          null;
    }
    Directory('${Directory.current.path}${separator}lib${separator}route')
        .createSync(recursive: true);
    if (guardMap.containsValue(true)) {
      //至少一个需要路由守卫，生成route_guard.route.dart文件
      File('${Directory.current.path}${separator}lib${separator}route${separator}route_guard.route.dart')
          .writeAsStringSync(tpl_guard.clazzTplGuardWidget(
              package: buildStep.inputId.package));
    } else {
      //删除已存在的文件
      File file = File(
          '${Directory.current.path}${separator}lib${separator}route${separator}route_guard.route.dart');
      if (file.existsSync()) {
        file.deleteSync();
      }
    }
    File('${Directory.current.path}${separator}lib${separator}route${separator}main.route.dart')
        .writeAsStringSync(tpl.clazzTpl(
            buildConstant: buildConstant(constantMap, defaultRoute, debugRoute),
            buildWidgetName: buildWidgetName(constantMap),
            buildRoutes: buildRoutes(routesMap, defaultRoute, guardMap),
            buildImport: buildImport(importMap),
            defaultRoute: defaultRoute,
            package: buildStep.inputId.package));
    return null;
  }
}

Builder routeBuilder(BuilderOptions options) =>
    LibraryBuilder(RouteGenerator(), generatedExtension: ".route.dart");
